@@ -38,6 +38,9 @@ SV* scalar_from_k(K k) {
             break;
 
         case KG: // byte
+            result = byte_from_k(k);
+            break;
+
         case KC: // char
             result = char_from_k(k);
             break;
@@ -99,10 +102,13 @@ SV* vector_from_k(K k) {
             break;
 
         case KG: // byte
-        case KC: // char
             result = byte_vector_from_k(k);
             break;
 
+        case KC: // char
+            result = char_vector_from_k(k);
+            break;
+
         case KH: // short
             result = short_vector_from_k(k);
             break;
@@ -165,7 +171,7 @@ SV* vector_from_k(K k) {
             break;
     }
 
-    return newRV_noinc((SV*)result);
+    return result;
 }
 
 /*
@@ -184,15 +190,47 @@ SV* xd_from_k(K k) {
     }
 }
 
+/* copy the contents of hv into store_hv */
+void hv_store_hv(HV *store_hv, HV *hv) {
+    int i;
+    int h_size = hv_iterinit(hv);
+    HE *store_ret, *he;
+    SV *key, *val;
+
+    for (i = 0; i < h_size; i++) {
+
+        he  = hv_iternext(hv);
+        key = hv_iterkeysv(he);
+        val = hv_iterval(hv, he);
+
+        store_ret = hv_store_ent(store_hv, key, val, 0);
+        if (store_ret == NULL) {
+            croak("Failed to store hash entry");
+        }
+
+        SvREFCNT_inc(val);
+    }
+}
+
 SV* ptable_from_k(K k) {
-    AV* av = newAV();
+    HV *hv = newHV();
+
     K t0   = kK(k)[0]; // partitioned tables have 2 sub-tables
     K t1   = kK(k)[1];
 
-    av_push(av, newRV_noinc( table_from_k(t0) ) );
-    av_push(av, newRV_noinc( table_from_k(t1) ) );
+    SV *t0_rv = table_from_k(t0);
+    SV *t1_rv = table_from_k(t1);
 
-    return (SV*) av;
+    HV *t0_hv = (HV*) SvRV( t0_rv );
+    HV *t1_hv = (HV*) SvRV( t1_rv );
+
+    hv_store_hv(hv, t0_hv);
+    hv_store_hv(hv, t1_hv);
+
+    SvREFCNT_dec(t0_rv);
+    SvREFCNT_dec(t1_rv);
+
+    return newRV_noinc( (SV*)hv );
 }
 
 SV* dict_from_k(K k) {
@@ -232,7 +270,7 @@ SV* dict_from_k(K k) {
     SvREFCNT_dec(keys_ref);
     SvREFCNT_dec(vals_ref);
 
-    return (SV*)hv;
+    return newRV_noinc( (SV*)hv );
 }
 
 SV* table_from_k(K k) {
@@ -263,11 +301,11 @@ SV* bool_from_k(K k) {
     return newSViv( k->g );
 }
 
-SV* char_from_k(K k) {
-    if (k->g == 0) {
-        return &PL_sv_undef;
-    }
+SV* byte_from_k(K k) {
+    return newSVuv( k->g );
+}
 
+SV* char_from_k(K k) {
     char byte_str[1];
     byte_str[0] = k->g;
     return newSVpvn(byte_str, 1);
@@ -378,10 +416,21 @@ SV* bool_vector_from_k(K k) {
         av_push(av, newSViv( kG(k)[i]) );
     }
 
-    return (SV*)av;
+    return newRV_noinc( (SV*)av );
 }
 
 SV* byte_vector_from_k(K k) {
+    AV *av = newAV();
+    int i = 0;
+
+    for (i = 0; i < k->n; i++) {
+        av_push(av, newSVuv( kG(k)[i] ));
+    }
+
+    return newRV_noinc( (SV*)av );
+}
+
+SV* char_vector_from_k(K k) {
     AV *av = newAV();
     char byte_str[1];
     int i = 0;
@@ -396,7 +445,7 @@ SV* byte_vector_from_k(K k) {
         av_push(av, newSVpvn(byte_str, 1));
     }
 
-    return (SV*)av;
+    return newRV_noinc( (SV*)av );
 }
 
 SV* short_vector_from_k(K k) {
@@ -422,7 +471,7 @@ SV* short_vector_from_k(K k) {
         av_push(av, newSViv( kH(k)[i]) );
     }
 
-    return (SV*)av;
+    return newRV_noinc( (SV*)av );
 }
 
 SV* int_vector_from_k(K k) {
@@ -448,7 +497,7 @@ SV* int_vector_from_k(K k) {
         av_push(av, newSViv( kI(k)[i]) );
     }
 
-    return (SV*)av;
+    return newRV_noinc( (SV*)av );
 }
 
 SV* long_vector_from_k(K k) {
@@ -474,7 +523,7 @@ SV* long_vector_from_k(K k) {
         av_push(av, newSVi64(kJ(k)[i]) );
     }
 
-    return (SV*)av;
+    return newRV_noinc( (SV*)av );
 }
 
 SV* timestamp_vector_from_k(K k) {
@@ -500,7 +549,7 @@ SV* timestamp_vector_from_k(K k) {
         av_push(av, newSVi64(kJ(k)[i]) );
     }
 
-    return (SV*)av;
+    return newRV_noinc( (SV*)av );
 }
 
 SV* real_vector_from_k(K k) {
@@ -516,7 +565,7 @@ SV* real_vector_from_k(K k) {
         av_push(av, newSVnv( kE(k)[i] ) );
     }
 
-    return (SV*)av;
+    return newRV_noinc( (SV*)av );
 }
 
 SV* float_vector_from_k(K k) {
@@ -532,7 +581,7 @@ SV* float_vector_from_k(K k) {
         av_push(av, newSVnv( kF(k)[i] ) );
     }
 
-    return (SV*)av;
+    return newRV_noinc( (SV*)av );
 }
 
 SV* symbol_vector_from_k(K k) {
@@ -551,5 +600,5 @@ SV* symbol_vector_from_k(K k) {
         av_push(av, newSVpv( kS(k)[i], 0 ) );
     }
 
-    return (SV*)av;
+    return newRV_noinc( (SV*)av );
 }
